package info.textgrid.middleware.test.online;

import static org.junit.Assert.assertTrue;
import java.io.IOException;
import java.util.Properties;
import org.apache.cxf.jaxrs.client.Client;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

/**
 * <p>
 * Some online tests for the TextGrid OAIMPH service --> verb=ListRecords <--
 * </p>
 * 
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2022-09-12
 * @since 2022-09-12
 */
@Ignore
public class TestListRecordsOnline {

  // **
  // FINALS
  // **

  // private static final String PROPERTIES_FILE = "oaipmh.test.textgridlab-org.properties";
  // TODO There is NO IDIOM test data for testing on dev.textgridlab.org! So eventually all
  // IDIOM tests will fail!
  protected static final String PROPERTIES_FILE = "oaipmh.test.dev-textgridlab-org.properties";

  // **
  // STATICS
  // **

  // Some JAXRS things.
  private static String oaipmhEndpoint;
  static Client oaipmhWebClient;

  // Properties
  private static String checkListRecordsDCFrom;
  private static String checkListRecordsDCUntil;
  private static String checkListRecordsDATACITEFrom;
  private static String checkListRecordsDATACITEUntil;
  private static String checkListRecordsSet;
  private static Integer checkListRecordsSetExpectedPages;
  private static Integer checkListRecordsSetExpectedPagesIDIOM;

  // **
  // PREPARATIONS
  // **

  /**
   * @throws Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception {

    // Get properties.
    Properties p = OAIPMHUtilitiesOnline.getPropertiesFromFile(PROPERTIES_FILE);

    // Set properties.
    oaipmhEndpoint = p.getProperty("oaipmhEndpoint");

    checkListRecordsDCFrom = p.getProperty("checkListRecordsDCFrom");
    checkListRecordsDCUntil = p.getProperty("checkListRecordsDCUntil");
    checkListRecordsDATACITEFrom = p.getProperty("checkListRecordsDATACITEFrom");
    checkListRecordsDATACITEUntil = p.getProperty("checkListRecordsDATACITEUntil");
    checkListRecordsSet = p.getProperty("checkListRecordsSet");
    checkListRecordsSetExpectedPages =
        Integer.parseInt(p.getProperty("checkListRecordsSetExpectedPages").trim());
    checkListRecordsSetExpectedPagesIDIOM =
        Integer.parseInt(p.getProperty("checkListRecordsSetExpectedPagesIDIOM").trim());

    // Get web client from endpoint.
    oaipmhWebClient = OAIPMHUtilitiesOnline.getOAIPMHWEebClient(oaipmhEndpoint);
  }

  /**
   * @throws Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception {
    //
  }

  /**
   * @throws Exception
   */
  @Before
  public void setUp() throws Exception {
    //
  }

  /**
   * @throws Exception
   */
  @After
  public void tearDown() throws Exception {
    //
  }

  // **
  // TESTS
  // **

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsSetDC() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#LISTRECORDS");

    int pages =
        OAIPMHUtilitiesOnline.testList(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.OAI_DC_PREFIX, checkListRecordsSet);

    if (pages != checkListRecordsSetExpectedPages) {
      assertTrue(pages + " != " + checkListRecordsSetExpectedPages, false);
    }
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsSetDATACITE() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testListRecordsSetDATACITE()");

    int pages =
        OAIPMHUtilitiesOnline.testList(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.OAI_DATACITE_PREFIX, checkListRecordsSet);

    if (pages != checkListRecordsSetExpectedPages) {
      assertTrue(pages + " != " + checkListRecordsSetExpectedPages, false);
    }
  }

  /**
   * <p>
   * Sets are not implemented in IDIOM requests, we always only ask for ONE project here!
   * </p>
   * 
   * @throws IOException
   */
  @Test
  @Ignore
  public void testListRecordsSetIDIOM() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testListRecordsSetIDIOM()");

    int pages =
        OAIPMHUtilitiesOnline.testList(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX, checkListRecordsSet);

    if (pages != checkListRecordsSetExpectedPagesIDIOM) {
      assertTrue(pages + " != " + checkListRecordsSetExpectedPagesIDIOM, false);
    }
  }

  /**
   * FIXME I don't understand this test. We do not have hdl: prefixes in TextGrid OAI-PMH, do we?
   * 
   * @throws IOException
   */
  @Test
  @Ignore
  public void testListRecordsSetDCDifferentPrefixes() throws IOException {

    String uri = checkListRecordsSet;

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#LISTRECORDS - HDL PREFIX");

    if (!uri.startsWith(OAIPMHUtilitiesOnline.HDL_PREFIX)) {
      assertTrue("missing '" + OAIPMHUtilitiesOnline.HDL_PREFIX + "' prefix!", false);
    }
    int pages1 = OAIPMHUtilitiesOnline.testList(oaipmhWebClient,
        OAIPMHUtilitiesOnline.VERB_LIST_RECORDS, OAIPMHUtilitiesOnline.OAI_DC_PREFIX, uri);
    if (pages1 != checkListRecordsSetExpectedPages) {
      System.out.println(pages1 + " != " + checkListRecordsSetExpectedPages);
    }

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#LISTRECORDS - NO PREFIX");

    if (checkListRecordsSet.startsWith(OAIPMHUtilitiesOnline.HDL_PREFIX)) {
      uri = checkListRecordsSet.substring(4);
    }
    int pages2 = OAIPMHUtilitiesOnline.testList(oaipmhWebClient,
        OAIPMHUtilitiesOnline.VERB_LIST_RECORDS, OAIPMHUtilitiesOnline.OAI_DC_PREFIX, uri);
    if (pages2 != checkListRecordsSetExpectedPages) {
      System.out.println(pages2 + " != " + checkListRecordsSetExpectedPages);
    }

    // Check pages count.
    if (pages1 != checkListRecordsSetExpectedPages || pages2 != checkListRecordsSetExpectedPages) {
      assertTrue("pages count does not match", false);
    }
  }

  /**
   * TODO Un-ignore if identifier check is implemented!
   * 
   * @throws IOException
   */
  @Test
  @Ignore
  public void testListRecordsSetDCStarPrefix() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#LISTRECORDS - STAR PREFIX");

    if (checkListRecordsSet.startsWith(OAIPMHUtilitiesOnline.HDL_PREFIX)) {
      checkListRecordsSet = OAIPMHUtilitiesOnline.STAR_PREFIX + checkListRecordsSet.substring(4);
    }

    int pages3 =
        OAIPMHUtilitiesOnline.testList(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
            OAIPMHUtilitiesOnline.OAI_DC_PREFIX, checkListRecordsSet);
    if (pages3 != 1) {
      System.out.println(pages3 + " != " + 1);
    }
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsDCFromUntil() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#LISTRECORDS");

    OAIPMHUtilitiesOnline.testList(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
        OAIPMHUtilitiesOnline.OAI_DC_PREFIX, OAIPMHUtilitiesOnline.NO_SET, checkListRecordsDCFrom,
        checkListRecordsDCUntil);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsDATASITE() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testListRecordsDATASITE()");

    OAIPMHUtilitiesOnline.testList(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
        OAIPMHUtilitiesOnline.OAI_DATACITE_PREFIX, OAIPMHUtilitiesOnline.NO_SET);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsIDIOM() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testListRecordsIDIOM()");

    OAIPMHUtilitiesOnline.testList(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
        OAIPMHUtilitiesOnline.OAI_IDIOMMETS_PREFIX, OAIPMHUtilitiesOnline.NO_SET);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsDC() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "testListRecordsDC()");

    OAIPMHUtilitiesOnline.testList(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
        OAIPMHUtilitiesOnline.OAI_DC_PREFIX, OAIPMHUtilitiesOnline.NO_SET);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsDATASITEFromUntil() throws IOException {

    System.out.println(OAIPMHUtilitiesOnline.TESTING + "#LISTRECORDS");

    OAIPMHUtilitiesOnline.testList(oaipmhWebClient, OAIPMHUtilitiesOnline.VERB_LIST_RECORDS,
        OAIPMHUtilitiesOnline.OAI_DATACITE_PREFIX, OAIPMHUtilitiesOnline.NO_SET,
        checkListRecordsDATACITEFrom, checkListRecordsDATACITEUntil);
  }

}
